unit mainfrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Grids, ComCtrls,
  SDL_vector, SDL_rchart, SDL_ntabed;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    BButLoadData: TBitBtn;
    BButPerformPCA: TBitBtn;
    ODiag1: TOpenDialog;
    PageControl1: TPageControl;
    TSData: TTabSheet;
    TSScores: TTabSheet;
    TabEdData: TNTabEd;
    RCScores: TRChart;
    StdDev: TVector;
    Mean: TVector;
    RGScaling: TRadioGroup;
    TSLoadings: TTabSheet;
    RCLoadings: TRChart;
    BButNewData: TBitBtn;
    procedure BButLoadDataClick(Sender: TObject);
    procedure BButPerformPCAClick(Sender: TObject);
    procedure BButNewDataClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  SDL_stringl, SDL_math2;

(******************************************************************************)
procedure TForm1.BButLoadDataClick(Sender: TObject);
(******************************************************************************)

begin
PageControl1.ActivePage := TSData;
ODiag1.FileName := '*.asc';
if ODiag1.Execute then
  TabEdData.Data.ImportASC (ODiag1.Filename);
end;


(******************************************************************************)
function MapColors(index: integer): TColor;
(******************************************************************************)

const
  DefColors : array[0..15] of TColor = (clBlack, clBlue, clRed, clGreen,
                                        clBlack, clBlue, clRed, clGreen,
                                        clBlack, clBlue, clRed, clGreen,
                                        clBlack, clBlue, clRed, clGreen);

begin
if index < 0 then
  index := 0;
index := index mod 16;
result := DefColors[index];
end;


(******************************************************************************)
procedure TForm1.BButPerformPCAClick(Sender: TObject);
(******************************************************************************)

const
  FIRSTPC = 1;
  SECONDPC = 2;

var
  i    : integer;
  m, s : double;
  x, y : double;
  obj  : integer;

begin
if PageControl1.ActivePage = TSData then
  PageControl1.ActivePage := TSScores;
if not CalcPrincComp (TabEdData.Data.NumericData, 1, TabEdData.Data.NrOfColumns, 1, TabEdData.Data.NrOfRows, RGScaling.ItemIndex)
  then MessageDlg ('Cannot calculate principal components', mtInformation, [mbOK], 0)
  else begin
       Mean.NrOfElem := TabEdData.NrOfColumns;
       StdDev.NrOfElem := TabEdData.NrOfColumns;  // calc. mean and standard deviation for correct score plot
       case RGScaling.ItemIndex of
          0 : begin        { scatter matrix }
              for i:=1 to TabEdData.NrOfColumns do
                begin
                Mean.Elem[i] := 0;
                StdDev.Elem[i] := 1;
                end;
              end;
          1 : begin       { covariance matrix }
              for i:=1 to TabEdData.NrOfColumns do
                begin
                TabEdData.Data.NumericData.MeanVar (i,1,i,TabEdData.NrOfRows,m,s);
                Mean.Elem[i] := m;
                StdDev.Elem[i] := 1;
                end;
              end;
          2 : begin       { correlation matrix }
              for i:=1 to TabEdData.NrOfColumns do
                begin
                TabEdData.Data.NumericData.MeanVar (i,1,i,TabEdData.NrOfRows,m,s);
                Mean.Elem[i] := m;
                StdDev.Elem[i] := sqrt(s);
                end;
              end;
       end;

           // display scores
       RCScores.ClearGraf;
       RCScores.Scale1X.Caption := 'PC '+IntToStr(FIRSTPC);
       RCScores.Scale1Y.Caption := 'PC '+IntToStr(SECONDPC);
       RCScores.Caption := 'Score/Score Plot - '+RGScaling.Items[RGScaling.Itemindex];
       for obj:=1 to TabEdData.NrOfRows do
         begin
         x := 0;
         for i:=1 to TabEdData.NrOfColumns do
           begin
           s := StdDev.Elem[i];
           if s <> 0.0 then
             x := x + GetEigenResult (FIRSTPC,i) * (TabEdData.Elem[i,obj]-Mean.Elem[i])/s;
           end;
         y := 0;
         for i:=1 to TabEdData.NrOfColumns do
           begin
           s := StdDev.Elem[i];
           if s <> 0.0 then
             y := y + GetEigenResult (SECONDPC,i) * (TabEdData.Elem[i,obj]-Mean.Elem[i])/s;
           end;
         RCScores.DataColor := MapColors(TabEdData.RowAttrib[obj]);
         RCScores.MarkAt (x,y,6);
         end;
       RCScores.AutoRange(1, 3);
       RCScores.ShowGraf;

              // display loadings
       RCLoadings.ClearGraf;
       RCLoadings.Scale1X.Caption := 'PC '+strff(FIRSTPC,1,0);
       RCLoadings.Scale1Y.Caption := 'PC '+strff(SECONDPC,1,0);
       RCLoadings.Caption := 'Loading/Loading Plot - '+RGScaling.Items[RGScaling.Itemindex];
       RCLoadings.DataColor := clGray;
       RCLoadings.Line (-1,0,1,0);
       RCLoadings.Line (0,-1,0,1);
       RCLoadings.DataColor := clNavy;
       RCLoadings.MoveTo (GetEigenResult (FIRSTPC,1),
                       GetEigenResult (SECONDPC,1));
       for i:=1 to TabEdData.NrOfColumns do
         begin
         RCLoadings.DataTag := 0;
         RCLoadings.Text (GetEigenResult (FIRSTPC,i), GetEigenResult (SECONDPC,i),8, IntToStr(i));
         end;
       RCLoadings.AutoRange(1, 3);
       RCLoadings.ShowGraf;
       end;
end;

(******************************************************************************)
procedure TForm1.BButNewDataClick(Sender: TObject);
(******************************************************************************)

var
  i, j : integer;

begin
TabEdData.SuppressPaint := true;
TabEdData.NrOfColumns := 5;
TabEdData.NrOfRows := 150;
for j:=1 to TabEdData.NrOfRows do
  begin
  for i:=1 to TabEdData.NrOfColumns do
    begin
    TabEdData.Data[i,j] := sqr(random) + (j mod 3)*(i mod 2);
    TabEdData.Data[i,j] := TabEdData.Data[i,j] + 3*TabEdData.Data[i-1,j]+0.1*(random);
    end;
  TabEdData.RowAttrib[j] := 1+ j mod 3;
  end;
TabEdData.SuppressPaint := false;
end;


(******************************************************************************)
procedure TForm1.FormShow(Sender: TObject);
(******************************************************************************)

begin
if SDL_ntabed.IsLightEd then
  BButLoadData.Enabled := false;
end;

end.
